// Copyright (C) 2001 Eric Hanchrow
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Eric Hanchrow -- offby1@blarg.net
// 1914 N. 40th
// Seatlle, WA, USA
// 98103-8310
#ifndef FILE_AUCTION_SEEN
#define FILE_AUCTION_SEEN

#include <vector>
#include "call.hh"
#include "compass.hh"
#include "contract.hh"

class auction
{
public:
  enum states
    {
      in_progress,
      passed_out,
      contract_settled
    };

private:
  bool _f_double_active;

  // These two are garbage if !_f_double_active
  bool _f_redouble;
  seat _doubler;

  std::vector<const call *> _calls;
  states _state;
  bid *_highest_bid;            // 0 if no bids yet
public:
  bool _f_someone_has_opened () const { return _highest_bid != 0; }
  seat dealer;
  bool ns_vulnerable;
  bool ew_vulnerable;
  auction () : _f_double_active (false),
               _state (in_progress),
               _highest_bid (0),
               ns_vulnerable (false),
               ew_vulnerable (false)
  {}
  ~auction ();

  /// C must be allocated with operator `new', because the dtor will
  /// delete it.
  void auction::push_back (const call *c);

  states get_state () const { return _state ; }

  // asserts, or does something equally nasty, if _state !=
  // contract_settled.
  contract get_contract () const;

  const bid * highest () const {return _highest_bid;}
  size_t size () const { return _calls.size (); }
  const std::vector<const call *> & calls () const { return _calls; }
};

#endif  /* FILE_AUCTION_SEEN */
