// Copyright (C) 2001 Eric Hanchrow
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Eric Hanchrow -- offby1@blarg.net
// 1914 N. 40th
// Seatlle, WA, USA
// 98103-8310
#ifndef FILE_CARD_SEEN
#define FILE_CARD_SEEN

#include <iostream>
#include <cassert>
#include "exceptions.hh"

enum rank
  {
    two = 2,
    three,
    four,
    five,
    six,
    seven,
    eight,
    nine,
    ten,
    jack,
    queen,
    king,
    ace
  };

enum suit
  {
    clubs = 0,
    diamonds,
    hearts,
    spades
  };

class card
{
public:
  rank get_rank () const  { return _r; };
  suit get_suit () const  { return _s; };

  unsigned short get_HCP () const; // Goren high-card point count

  card (unsigned short index) :
    _r (static_cast<rank>(two   + (index % 13))),
    _s (static_cast<suit>(clubs + (index / 13)))
  {
    Assert<Bad_Card> (_r <= ace);
    Assert<Bad_Card> (_s <= spades);
  }

private:
  card () {}
  rank _r;
  suit _s;
};

std::ostream & operator<< (std::ostream &o, const rank &r);
std::ostream & operator<< (std::ostream &o, const suit &s);
std::ostream & operator<< (std::ostream &o, const card &c);

#endif  /* FILE_CARD_SEEN */
