// Copyright (C) 2001 Eric Hanchrow
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Eric Hanchrow -- offby1@blarg.net
// 1914 N. 40th
// Seatlle, WA, USA
// 98103-8310
#include <iostream>
#include "dumb_player.hh"
#include "../auction.hh"
#include "../hand.hh"
#include "../score.hh"

namespace {
call *maybe_open (const seat &who_am_i,
                  const auction &so_far,
                  const hand &h)
{
  call *return_value = 0;

  unsigned short length_points = 0;
  unsigned short five_card_majors = 0;

  if (h.length (spades) > 4)
    {
      length_points += h.length (spades) - 4;
      five_card_majors++;
    }

  if (h.length (hearts) > 4)
    {
      length_points += h.length (hearts) - 4;
      five_card_majors++;
    }

  if (h.length (diamonds) > 4)
    length_points += h.length (diamonds) - 4;

  if (h.length (clubs) > 4)
    length_points += h.length (clubs) - 4;

  if (h.is_balanced ()
      &&
      h.HCP () >= 16
      &&
      h.HCP () <= 18)
    {
      denomination notrump;
      return_value = new bid (1, notrump);
    }
  else if (h.is_balanced ()
           &&
           h.HCP () >= 22
           &&
           h.HCP () <= 24)
    {
      denomination notrump;
      return_value = new bid (2, notrump);
    }
  else if (h.is_balanced ()
           &&
           h.HCP () >= 25
           &&
           h.HCP () <= 27)
    {
      denomination notrump;
      return_value = new bid (3, notrump);
    }
  else
    {
      const unsigned short points = h.HCP () + length_points;

      suit suit_to_bid        = hearts;

      if ((h.length (spades) > h.length (hearts)))
        suit_to_bid = spades;

      if (h.length (suit_to_bid) <= 4)
        {
          if (h.length (clubs) > h.length (diamonds))
            suit_to_bid = clubs;
          else
            suit_to_bid = diamonds;
        }

      if (points < 13)
        {
          return_value = new pass;
        }
      else if (points < 22)
        {
          return_value = new bid (1, suit_to_bid);
        }
      else
        {
          return_value = new bid (2, suit_to_bid);
        }
    }
  assert (return_value);
  return return_value;
}
};

call *dumb_player::make_call (const seat &who_am_i, // only for diagnostics and ruminations
                              const auction &so_far,
                              const rubber_scoresheet &score)
{
  call *return_value = 0;

  if (so_far._f_someone_has_opened ())
    {
      const Double *previous_double = dynamic_cast<const Double *>(so_far.calls().back ());
      if (previous_double
          &&
          (2 == previous_double->multiplier ()))
        {
          return_value = new Double (true);
        }
      else if ((rand () % 4 > 0)
          &&
          so_far.highest ()
          &&
          ! (so_far.highest ()-> denom ().is_notrump ()
             &&
             so_far.highest ()-> level () == 7))
        return_value = new bid (so_far.highest ()->next ());
      else
        {
          // Previous (i.e., right-hand opponent's) call was a bid, so
          // I can double it.  And the way this program bids, it's not
          // a bad idea :)
          if ((dynamic_cast<const bid *>(so_far.calls().back ()))
              &&
              (rand () % 4 == 0))
            {
              return_value = new Double;
            }
          else
            return_value = new pass;
        }

    }
  else
    return_value = maybe_open (who_am_i,
                               so_far,
                               _hand);

  return return_value;
}

