// Copyright (C) 2001 Eric Hanchrow
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Eric Hanchrow -- offby1@blarg.net
// 1914 N. 40th
// Seatlle, WA, USA
// 98103-8310
#include <vector>
#include "../hand.hh"
#include "../auction.hh"
#include "../card.hh"
#include "dumb_player.hh"
#include "../score.hh"

/// Rudimentary card-playing algorithm.

/**
 * Plays the first card it finds in the suit led, unless there are no
 * cards in that suit, in which case it simply plays the first card it
 * finds.
 *
 * Currently ignores most of its inputs, because it's dumb.  But a
 * good player would take into account the auction, partscores,
 * vulnerability, and the previous cards played this hand.
 */
card dumb_player::choose_card (const auction &,
                               const rubber_scoresheet &,
                               const plays &,
                               const std::vector<card> &current_trick,
                               const hand *dummy // will be 0 for opening lead
                               )
{
  Assert<internal_error> (_hand.size () > 0, "Can't choose a card when my hand is empty");

  std::vector<card>::iterator choice = _hand.begin ();

  // If no cards have yet been played to the current trick, no need to
  // follow suit; we'll just be dumb and return the first card in the
  // hand.

  if (current_trick.size () > 0)
    {
      const suit &led = current_trick[0].get_suit ();

      // Return the first card of that suit.

      for (;
           choice != _hand.end ();
           choice++)
        {
          if (choice->get_suit () == led)
            break;
        }

      // Can't follow suit?  Then play any old card.
      if (choice == _hand.end ())
        choice = _hand.begin ();
    }

  card return_value = *choice;

  // Once I play a card, I no longer hold it.
  _hand.erase (choice);

  return return_value;
}
