// Copyright (C) 2001 Eric Hanchrow
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Eric Hanchrow -- offby1@blarg.net
// 1914 N. 40th
// Seatlle, WA, USA
// 98103-8310
#ifndef FILE_HAND_SEEN
#define FILE_HAND_SEEN

#include <vector>
#include <algorithm>
#include "card.hh"

class hand : public std::vector<card>
{
public:
  hand () {}
  hand (std::vector<card>::const_iterator start, std::vector<card>::const_iterator end)
    : std::vector<card> (start, end)
  {
  }
  unsigned short HCP () const
  {
    unsigned short total = 0;
    for (std::vector<card>::const_iterator i = this->begin ();
         i != this->end ();
         i++)
      {
        total += i->get_HCP ();
      }
    return total;
  }

  unsigned short length (const suit s) const
  {
    unsigned short total = 0;
    for (std::vector<card>::const_iterator i = this->begin ();
         i != this->end ();
         i++)
      {
        if (s == i->get_suit ())
          {
            total++;
          }
      }
    return total;
  }

  bool is_balanced () const
  {
    std::vector<unsigned short>lengths;
    lengths.push_back (length (spades  ));
    lengths.push_back (length (hearts  ));
    lengths.push_back (length (diamonds));
    lengths.push_back (length (clubs   ));

    std::sort (lengths.begin (), lengths.end ());

    // Singleton or void?
    if (lengths[0] < 2)
      return false;

    // Six-card suit?
    if (lengths[3] > 5)
      return false;

    // Two doubletons?
    if (lengths [1] == 2)
      return false;

    return true;
  }

  void print (std::ostream &o, bool display_bidding_info=false) const;
};

#endif  /* FILE_HAND_SEEN */
