// Copyright (C) 2001 Eric Hanchrow
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Eric Hanchrow -- offby1@blarg.net
// 1914 N. 40th
// Seatlle, WA, USA
// 98103-8310
#include "score.hh"
#include "hand.hh"
#include "call.hh"
#include "trick.hh"
#include "contract.hh"

void rubber_scoresheet::tally_hand (const plays &p,
                                    const contract &c)
{
  std::cout << "OK, that hand's over.  "
            << c
            << std::endl;

  // Count tricks taken by declarer

  // Avoid the temptation to declare this, and anything else in this
  // function, as a short instead of a plain int.  If you do so, some
  // of the math below (in particular, multiplication of
  // number-of-tricks by penalty-points-per-trick) may be wrong due to
  // overflow.  Ah, C++.
  int declarer_odd_tricks = -6;

  const bool ns_were_declarer = (0 == (c.get_declarer ().offset_from_north () % 2));

  const bool declarer_vulnerable  =   (ns_were_declarer  && this->ns_vulnerable ()
                                       ||
                                       !ns_were_declarer && this->ew_vulnerable ());

  for (plays::const_iterator i = p.begin ();
       i != p.end ();
       i++)
    {
      // Ugly, eh?  In English, it means "if the trick's winner and
      // the declarer are separated by 0 or 2 seats, then the
      // declarer's team won the trick".
      if (0 == ((i->winner () - c.get_declarer ())
                % 2))
        declarer_odd_tricks++;
    }

  unsigned int multiplier = 1;
  switch (c.get_multiplier ())
    {
    case contract::doubled:
      multiplier = 2;
      break;
    case contract::redoubled:
      multiplier = 4;
      break;
    default:
      break;
    }

  std::cout << "This contract was "
            << ((multiplier == 4)
                ? "redoubled"
                : ((multiplier == 2)
                   ? "doubled"
                   : "not doubled"))
            << std::endl;

  // These refer to points earned *this hand*, not the total points
  // earned in previous hands in this rubber -- that value is already
  // present in the columns.
  unsigned int below_the_line = 0;
  unsigned int declarer_above = 0;
  unsigned int defender_above = 0;

  const int overtricks = declarer_odd_tricks - c.get_bid().level ();
  const denomination dn = c.get_bid ().denom ();

  // Did declarer make contract?

  if (overtricks >= 0)
    {
      std::cout << "Woo hoo!  Declarer made the contract."
                << std::endl;

      // Figure below-the-line points, and perhaps above-the-line
      // points, for declarer

      if (dn.is_notrump ())
        below_the_line = (10 + 30 * (declarer_odd_tricks - overtricks)) * multiplier;
      else
        switch (dn.get_suit ())
          {
          case spades: case hearts:
            below_the_line = 30 * (declarer_odd_tricks - overtricks) * multiplier;
            break;
          default:
            below_the_line = 20 * (declarer_odd_tricks - overtricks) * multiplier;
            break;
          }

      std::cout << "Got " << below_the_line << " points below the line ..."
                << std::endl;

      // Ways to get above-the-line points:

      // overtricks
      if (overtricks > 0)
        {
          std::cout << "Woo hoo ^ 2!  Declarer got "
                    << overtricks
                    << " overtricks! ...";

          if (c.get_multiplier ()== contract::not_doubled)
            {
              if (dn.is_notrump ())
                {
                  declarer_above += 30 * overtricks;
                }
              else
                switch (dn.get_suit ())
                  {
                  case clubs: case diamonds:
                    declarer_above += 20 * overtricks;
                    break;
                  default:
                    declarer_above += 30 * overtricks;
                    break;
                  }
            }
          else
            {
              declarer_above += 50 * overtricks  * multiplier * (declarer_vulnerable
                                                                 ? 2
                                                                 : 1);
            }

          std::cout << " for " << declarer_above << " above-the-line points."
                    << std::endl;
        }

      // contract was doubled
      switch (c.get_multiplier ())
        {
        case contract::doubled:
          declarer_above += 50;
          break;
        case contract::redoubled:
          declarer_above += 100;
          break;
        default:
          break;
        }

      // contract was a slam
      switch (c.get_bid ().level ())
        {
        case 6:                 // small slam
          declarer_above += (declarer_vulnerable
                             ? 750
                             : 500);
          std::cout << "Yippee!  Small slam." << std::endl;
          break;
        case 7:                 // grand slam
          declarer_above += (declarer_vulnerable
                             ? 1500
                             : 1000);
          std::cout << "Yippee!  Grand slam." << std::endl;
          break;
        default:
          break;
        }

      // rubber bonus will be figured by column::add_below.
    }
  else
    // Figure above-the-line points for defenders
    {
      std::cout << "Down " << -overtricks << std::endl;

      // Undertrick points
      // Good Lord.  This is horribly complicated; no wonder I never
      // learned how to keep score.
      if (!declarer_vulnerable)
        switch (c.get_multiplier ())
          {
          case contract::redoubled:
            switch (overtricks)
              {
                // Note the fallthrough!
              case -13:
                defender_above += 600;
              case -12:
                defender_above += 600;
              case -11:
                defender_above += 600;
              case -10:
                defender_above += 600;
              case -9:
                defender_above += 600;
              case -8:
                defender_above += 600;
              case -7:
                defender_above += 600;
              case -6:
                defender_above += 600;
              case -5:
                defender_above += 600;
              case -4:
                defender_above += 600;
              case -3:
                defender_above += 400;
              case -2:
                defender_above += 400;
              case -1:
                defender_above += 200;
              }
            break;
          case contract::doubled:
            switch (overtricks)
              {
                // Note the fallthrough!
              case -13:
                defender_above += 300;
              case -12:
                defender_above += 300;
              case -11:
                defender_above += 300;
              case -10:
                defender_above += 300;
              case -9:
                defender_above += 300;
              case -8:
                defender_above += 300;
              case -7:
                defender_above += 300;
              case -6:
                defender_above += 300;
              case -5:
                defender_above += 300;
              case -4:
                defender_above += 300;
              case -3:
                defender_above += 200;
              case -2:
                defender_above += 200;
              case -1:
                defender_above += 100;
              }
            break;
          default:
            // Note overtricks is < 0, so we multiply by *negative*
            // fifty to make the bonus positive.
            defender_above += -50 * overtricks;
            break;
          }
      else                      // declarer vulnerable
        switch (c.get_multiplier ())
          {
          case contract::redoubled:
            defender_above += 400;
            if (overtricks < -1)
              defender_above += 600 * (overtricks * -1);
            break;
          case contract::doubled:
            defender_above += 200;
            if (overtricks < -1)
              defender_above += 300 * (overtricks * -1);
            break;
          default:
            defender_above += -100 * overtricks;
            break;
          }

      std::cout << "Uh oh!  Declarer didn't make it; defenders get "
                << defender_above
                << " points above the line."
                << std::endl;
    }

  if (ns_were_declarer)
    {
      // We have to capture this bit now, because `add_below' may
      // clobber it.
      bool they_wuz_vulnerable = this->ns_vulnerable ();

      _north_south.add_below (below_the_line,
                              _east_west.is_vulnerable ());

      // This is a little obscure.  We're trying to tell if they just
      // won the rubber; we can tell that they have if:
      //
      // 1) they were vulnerable;
      //
      // 2) if the below_the_line score is 0;
      //
      // 3) we just added > 0 points below the line.  below_the_line
      // drops to zero when a team wins a game, because the points
      // suddenly become above-the-line (because the scorekeeper drew
      // a line under the old below-the-line score :-)
      if (they_wuz_vulnerable
          &&
          (below_the_line > 0)
          &&
          (0 == _north_south.below ()))
        _completed = true;

      _north_south.add_above (declarer_above);
      _east_west  .add_above (defender_above);
    }
  else
    {
      bool they_wuz_vulnerable = this->ew_vulnerable ();

      _east_west  .add_below (below_the_line,
                              _north_south.is_vulnerable ());

      if (they_wuz_vulnerable
          &&
          (below_the_line > 0)
          &&
          (0 == _east_west.below ()))
        _completed = true;

      _east_west  .add_above (declarer_above);
      _north_south.add_above (defender_above);
    }
}

namespace {

  unsigned short aces (const hand &h)
  {
    unsigned short retval = 0;
    for (std::vector<card>::const_iterator i = h.begin ();
         i != h.end ();
         i++)
      {
        if (i->get_rank ()== ace)
          retval++;
      }

    return retval;
  }

  unsigned short honors (const suit &s,
                         const hand &h)
  {
    unsigned short retval = 0;
    for (std::vector<card>::const_iterator i = h.begin ();
         i != h.end ();
         i++)
      {
        if ((i->get_rank () >= ten)
            &&
            (i->get_suit () == s))
          retval++;
      }
    return retval;
  }
};

void rubber_scoresheet::credit_honors (const contract &c,
                                       const hand &declarer,
                                       const hand &dummy,
                                       const hand &lho,
                                       const hand &rho)
{
  const denomination &d = c.get_bid ().denom ();
  const bool ns_were_declarer = (0 == (c.get_declarer ().offset_from_north () % 2));
  column & declarer_column = (ns_were_declarer
                              ? _north_south
                              : _east_west);
  column & defenders_column= (ns_were_declarer
                              ? _east_west
                              : _north_south);

  if (d.is_notrump ())
    {
      if (4 == aces (declarer)
          ||
          4 == aces (dummy))
        {
          std::cout << "Declarer gets 150 points for notrump honors." << std::endl;
          declarer_column.add_above (150);
        }
      else if (4 == aces (lho)
               ||
               4 == aces (rho))
        {
          std::cout << "Defenders get 150 points for notrump honors." << std::endl;
          defenders_column.add_above (150);
        }
    }
  else
    {
      const suit s = d.get_suit ();
      // if one hand had four of five trump honors (A, K, Q, J, 10),
      // 100 points; 150 if all five
      if ((4 == honors (s, declarer))
          ||
          (4 == honors (s, dummy)))
        {
          std::cout << "Declarer gets 100 points for trump honors." << std::endl;
          declarer_column.add_above (100);
        }
      else if ((5 == honors (s, declarer))
               ||
               (5 == honors (s, dummy)))
        {
          std::cout << "Declarer gets 150 points for trump honors." << std::endl;
          declarer_column.add_above (150);
        }

      // One would hope the defenders don't have trump honors, but you
      // never know :)

      if ((4 == honors (s, lho))
          ||
          (4 == honors (s, rho)))
        {
          std::cout << "Yikes. Defenders get 100 points for trump honors." << std::endl;
          defenders_column.add_above (100);
        }
      else if ((5 == honors (s, lho))
               ||
               (5 == honors (s, rho)))
        {
          std::cout << "Yikes! Defenders get 150 points for trump honors." << std::endl;
          defenders_column.add_above (150);
        }
    }
}

void column::add_above (unsigned short s)
{
  // Just add 'em.
  this->_above += s;
}

void column::add_below (unsigned short s,
                        bool opponents_vulnerable)
{
  this->_below += s;

  // Now if we hit 100 points, that's a game ...
  if (this->below () >= 100)
    {
      this->_above += this->_below;
      this->_below = 0;

      if (this->is_vulnerable ())
        {
          // Rubber bonus
          if (opponents_vulnerable)
            this->_above += 500;
          else
            this->_above += 700;
        }

      this->_vulnerable = true;
    }
}
